@@ -128,13 +128,21 @@ static char *get_reg_profile(RzAnalysis *analysis) {
 	return strdup(p);
 }
 
-static int archinfo(RzAnalysis *analysis, int query) {
-	if (query == RZ_ANALYSIS_ARCHINFO_MIN_OP_SIZE) {
+static int archinfo(RzAnalysis *a, RzAnalysisInfoType query) {
+	switch (query) {
+	case RZ_ANALYSIS_ARCHINFO_MIN_OP_SIZE:
 		return 1;
-	} else if (query == RZ_ANALYSIS_ARCHINFO_MAX_OP_SIZE) {
+	case RZ_ANALYSIS_ARCHINFO_MAX_OP_SIZE:
 		return 16;
+	case RZ_ANALYSIS_ARCHINFO_TEXT_ALIGN:
+		/* fall-thru */
+	case RZ_ANALYSIS_ARCHINFO_DATA_ALIGN:
+		return 0;
+	case RZ_ANALYSIS_ARCHINFO_CAN_USE_POINTERS:
+		return false;
+	default:
+		return -1;
 	}
-	return 0;
 }
 
 static bool java_init(void **user) {
