@@ -6,6 +6,8 @@
 #include <capstone/capstone.h>
 #include <capstone/ppc.h>
 #include "../../asm/arch/ppc/libvle/vle.h"
+#include "../arch/ppc/ppc_analysis.h"
+#include "../arch/ppc/ppc_il.h"
 
 #define SPR_HID0 0x3f0 /* Hardware Implementation Register 0 */
 #define SPR_HID1 0x3f1 /* Hardware Implementation Register 1 */
@@ -20,10 +22,6 @@ struct Getarg {
 	int bits;
 };
 
-#define INSOPS   insn->detail->ppc.op_count
-#define INSOP(n) insn->detail->ppc.operands[n]
-#define IMM(x)   (ut64)(insn->detail->ppc.operands[x].imm)
-
 #ifndef PFMT32x
 #define PFMT32x "lx"
 #endif
@@ -209,12 +207,11 @@ static void opex(RzStrBuf *buf, csh handle, cs_insn *insn) {
 
 static char *get_reg_profile(RzAnalysis *analysis) {
 	const char *p = NULL;
-	if (analysis->bits == 32) {
+	if (analysis->bits == 64) {
 		p =
 			"=PC	pc\n"
 			"=SP	r1\n"
 			"=BP	r31\n"
-			"=SR	srr1\n" // status register ??
 			"=SN	r3\n" // also for ret
 			"=A0	r3\n" // also for ret
 			"=A1	r4\n"
@@ -223,99 +220,259 @@ static char *get_reg_profile(RzAnalysis *analysis) {
 			"=A4	r7\n"
 			"=A5	r8\n"
 			"=A6	r6\n"
-			"gpr	srr0   .32 0   0\n"
-			"gpr	srr1   .32 4   0\n"
-			"gpr	r0   .32 8   0\n"
-			"gpr	r1   .32 12  0\n"
-			"gpr	r2   .32 16  0\n"
-			"gpr	r3   .32 20  0\n"
-			"gpr	r4   .32 24  0\n"
-			"gpr	r5   .32 28  0\n"
-			"gpr	r6   .32 32  0\n"
-			"gpr	r7   .32 36  0\n"
-			"gpr	r8   .32 40  0\n"
-			"gpr	r9   .32 44  0\n"
-			"gpr	r10 .32 48  0\n"
-			"gpr	r11 .32 52  0\n"
-			"gpr	r12 .32 56  0\n"
-			"gpr	r13 .32 60  0\n"
-			"gpr	r14 .32 64  0\n"
-			"gpr	r15 .32 68  0\n"
-			"gpr	r16 .32 72  0\n"
-			"gpr	r17 .32 76  0\n"
-			"gpr	r18 .32 80  0\n"
-			"gpr	r19 .32 84  0\n"
-			"gpr	r20 .32 88  0\n"
-			"gpr	r21 .32 92  0\n"
-			"gpr	r22 .32 96  0\n"
-			"gpr	r23 .32 100 0\n"
-			"gpr	r24 .32 104 0\n"
-			"gpr	r25 .32 108 0\n"
-			"gpr	r26 .32 112 0\n"
-			"gpr	r27 .32 116 0\n"
-			"gpr	r28 .32 120 0\n"
-			"gpr	r29 .32 124 0\n"
-			"gpr	r30 .32 128 0\n"
-			"gpr	r31 .32 132 0\n"
-			"gpr	lr   .32 136 0\n"
-			"gpr	ctr .32 140 0\n"
-			"gpr	msr .32 144 0\n"
-			"gpr	pc   .32 148 0\n"
-			"gpr	cr  .64 152 0\n"
-			"gpr	cr0 .8  152 0\n"
-			"gpr	cr1 .8  153 0\n"
-			"gpr	cr2 .8  154 0\n"
-			"gpr	cr3 .8  155 0\n"
-			"gpr	cr4 .8  156 0\n"
-			"gpr	cr5 .8  157 0\n"
-			"gpr	cr6 .8  158 0\n"
-			"gpr	cr7 .8  159 0\n"
-			"gpr	xer .32 160 0\n"
-			"gpr	mq   .32 164 0\n"
-			"gpr	fpscr  .32 168 0\n"
-			"gpr	vrsave .32 172 0\n"
-			"gpr	pvr .32 176 0\n"
-			"gpr	dccr   .32 180 0\n"
-			"gpr	iccr   .32 184 0\n"
-			"gpr	dear   .32 188 0\n"
-			"gpr	hid0   .32 192 0\n"
-			"gpr	hid1   .32 196 0\n"
-			"gpr	hid2   .32 200 0\n"
-			"gpr	hid3   .32 204 0\n"
-			"gpr	hid4   .32 208 0\n"
-			"gpr	hid5   .32 212 0\n"
-			"gpr	hid6   .32 216 0\n"
-			"gpr	ibat0  .64 220 0\n"
-			"gpr	ibat1  .64 228 0\n"
-			"gpr	ibat2  .64 236 0\n"
-			"gpr	ibat3  .64 244 0\n"
-			"gpr	ibat0l .32 220 0\n"
-			"gpr	ibat1l .32 228 0\n"
-			"gpr	ibat2l .32 236 0\n"
-			"gpr	ibat3l .32 244 0\n"
-			"gpr	ibat0u .32 224 0\n"
-			"gpr	ibat1u .32 232 0\n"
-			"gpr	ibat2u .32 240 0\n"
-			"gpr	ibat3u .32 248 0\n"
-			"gpr	dbat0  .64 256 0\n"
-			"gpr	dbat1  .64 264 0\n"
-			"gpr	dbat2  .64 272 0\n"
-			"gpr	dbat3  .64 280 0\n"
-			"gpr	dbat0l .32 256 0\n"
-			"gpr	dbat1l .32 264 0\n"
-			"gpr	dbat2l .32 272 0\n"
-			"gpr	dbat3l .32 280 0\n"
-			"gpr	dbat0u .32 260 0\n"
-			"gpr	dbat1u .32 268 0\n"
-			"gpr	dbat2u .32 276 0\n"
-			"gpr	dbat3u .32 284 0\n"
-			"gpr	mask   .32 288 0\n";
+			"=OF	ov\n"
+			"=CF	ca\n"
+
+			"gpr	r0	.64	0	0	\n"
+			"gpr	r1	.64	8	0	\n"
+			"gpr	r2	.64	16	0	\n"
+			"gpr	r3	.64	24	0	\n"
+			"gpr	r4	.64	32	0	\n"
+			"gpr	r5	.64	40	0	\n"
+			"gpr	r6	.64	48	0	\n"
+			"gpr	r7	.64	56	0	\n"
+			"gpr	r8	.64	64	0	\n"
+			"gpr	r9	.64	72	0	\n"
+			"gpr	r10	.64	80	0	\n"
+			"gpr	r11	.64	88	0	\n"
+			"gpr	r12	.64	96	0	\n"
+			"gpr	r13	.64	104	0	\n"
+			"gpr	r14	.64	112	0	\n"
+			"gpr	r15	.64	120	0	\n"
+			"gpr	r16	.64	128	0	\n"
+			"gpr	r17	.64	136	0	\n"
+			"gpr	r18	.64	144	0	\n"
+			"gpr	r19	.64	152	0	\n"
+			"gpr	r20	.64	160	0	\n"
+			"gpr	r21	.64	168	0	\n"
+			"gpr	r22	.64	176	0	\n"
+			"gpr	r23	.64	184	0	\n"
+			"gpr	r24	.64	192	0	\n"
+			"gpr	r25	.64	200	0	\n"
+			"gpr	r26	.64	208	0	\n"
+			"gpr	r27	.64	216	0	\n"
+			"gpr	r28	.64	224	0	\n"
+			"gpr	r29	.64	232	0	\n"
+			"gpr	r30	.64	240	0	\n"
+			"gpr	r31	.64	248	0	\n"
+			"vc	vs0	.128	256	0	\n"
+			"vc	vs1	.128	272	0	\n"
+			"vc	vs2	.128	288	0	\n"
+			"vc	vs3	.128	304	0	\n"
+			"vc	vs4	.128	320	0	\n"
+			"vc	vs5	.128	336	0	\n"
+			"vc	vs6	.128	352	0	\n"
+			"vc	vs7	.128	368	0	\n"
+			"vc	vs8	.128	384	0	\n"
+			"vc	vs9	.128	400	0	\n"
+			"vc	vs10	.128	416	0	\n"
+			"vc	vs11	.128	432	0	\n"
+			"vc	vs12	.128	448	0	\n"
+			"vc	vs13	.128	464	0	\n"
+			"vc	vs14	.128	480	0	\n"
+			"vc	vs15	.128	496	0	\n"
+			"vc	vs16	.128	512	0	\n"
+			"vc	vs17	.128	528	0	\n"
+			"vc	vs18	.128	544	0	\n"
+			"vc	vs19	.128	560	0	\n"
+			"vc	vs20	.128	576	0	\n"
+			"vc	vs21	.128	592	0	\n"
+			"vc	vs22	.128	608	0	\n"
+			"vc	vs23	.128	624	0	\n"
+			"vc	vs24	.128	640	0	\n"
+			"vc	vs25	.128	656	0	\n"
+			"vc	vs26	.128	672	0	\n"
+			"vc	vs27	.128	688	0	\n"
+			"vc	vs28	.128	704	0	\n"
+			"vc	vs29	.128	720	0	\n"
+			"vc	vs30	.128	736	0	\n"
+			"vc	vs31	.128	752	0	\n"
+			"vc	vs32	.128	768	0	\n"
+			"vc	vs33	.128	784	0	\n"
+			"vc	vs34	.128	800	0	\n"
+			"vc	vs35	.128	816	0	\n"
+			"vc	vs36	.128	832	0	\n"
+			"vc	vs37	.128	848	0	\n"
+			"vc	vs38	.128	864	0	\n"
+			"vc	vs39	.128	880	0	\n"
+			"vc	vs40	.128	896	0	\n"
+			"vc	vs41	.128	912	0	\n"
+			"vc	vs42	.128	928	0	\n"
+			"vc	vs43	.128	944	0	\n"
+			"vc	vs44	.128	960	0	\n"
+			"vc	vs45	.128	976	0	\n"
+			"vc	vs46	.128	992	0	\n"
+			"vc	vs47	.128	1008	0	\n"
+			"vc	vs48	.128	1024	0	\n"
+			"vc	vs49	.128	1040	0	\n"
+			"vc	vs50	.128	1056	0	\n"
+			"vc	vs51	.128	1072	0	\n"
+			"vc	vs52	.128	1088	0	\n"
+			"vc	vs53	.128	1104	0	\n"
+			"vc	vs54	.128	1120	0	\n"
+			"vc	vs55	.128	1136	0	\n"
+			"vc	vs56	.128	1152	0	\n"
+			"vc	vs57	.128	1168	0	\n"
+			"vc	vs58	.128	1184	0	\n"
+			"vc	vs59	.128	1200	0	\n"
+			"vc	vs60	.128	1216	0	\n"
+			"vc	vs61	.128	1232	0	\n"
+			"vc	vs62	.128	1248	0	\n"
+			"vc	vs63	.128	1264	0	\n"
+			"fpu	f0	.64	1280	0	\n"
+			"fpu	f1	.64	1288	0	\n"
+			"fpu	f2	.64	1296	0	\n"
+			"fpu	f3	.64	1304	0	\n"
+			"fpu	f4	.64	1312	0	\n"
+			"fpu	f5	.64	1320	0	\n"
+			"fpu	f6	.64	1328	0	\n"
+			"fpu	f7	.64	1336	0	\n"
+			"fpu	f8	.64	1344	0	\n"
+			"fpu	f9	.64	1352	0	\n"
+			"fpu	f10	.64	1360	0	\n"
+			"fpu	f11	.64	1368	0	\n"
+			"fpu	f12	.64	1376	0	\n"
+			"fpu	f13	.64	1384	0	\n"
+			"fpu	f14	.64	1392	0	\n"
+			"fpu	f15	.64	1400	0	\n"
+			"fpu	f16	.64	1408	0	\n"
+			"fpu	f17	.64	1416	0	\n"
+			"fpu	f18	.64	1424	0	\n"
+			"fpu	f19	.64	1432	0	\n"
+			"fpu	f20	.64	1440	0	\n"
+			"fpu	f21	.64	1448	0	\n"
+			"fpu	f22	.64	1456	0	\n"
+			"fpu	f23	.64	1464	0	\n"
+			"fpu	f24	.64	1472	0	\n"
+			"fpu	f25	.64	1480	0	\n"
+			"fpu	f26	.64	1488	0	\n"
+			"fpu	f27	.64	1496	0	\n"
+			"fpu	f28	.64	1504	0	\n"
+			"fpu	f29	.64	1512	0	\n"
+			"fpu	f30	.64	1520	0	\n"
+			"fpu	f31	.64	1528	0	\n"
+			"vc	v0	.128	1536	0	\n"
+			"vc	v1	.128	1552	0	\n"
+			"vc	v2	.128	1568	0	\n"
+			"vc	v3	.128	1584	0	\n"
+			"vc	v4	.128	1600	0	\n"
+			"vc	v5	.128	1616	0	\n"
+			"vc	v6	.128	1632	0	\n"
+			"vc	v7	.128	1648	0	\n"
+			"vc	v8	.128	1664	0	\n"
+			"vc	v9	.128	1680	0	\n"
+			"vc	v10	.128	1696	0	\n"
+			"vc	v11	.128	1712	0	\n"
+			"vc	v12	.128	1728	0	\n"
+			"vc	v13	.128	1744	0	\n"
+			"vc	v14	.128	1760	0	\n"
+			"vc	v15	.128	1776	0	\n"
+			"vc	v16	.128	1792	0	\n"
+			"vc	v17	.128	1808	0	\n"
+			"vc	v18	.128	1824	0	\n"
+			"vc	v19	.128	1840	0	\n"
+			"vc	v20	.128	1856	0	\n"
+			"vc	v21	.128	1872	0	\n"
+			"vc	v22	.128	1888	0	\n"
+			"vc	v23	.128	1904	0	\n"
+			"vc	v24	.128	1920	0	\n"
+			"vc	v25	.128	1936	0	\n"
+			"vc	v26	.128	1952	0	\n"
+			"vc	v27	.128	1968	0	\n"
+			"vc	v28	.128	1984	0	\n"
+			"vc	v29	.128	2000	0	\n"
+			"vc	v30	.128	2016	0	\n"
+			"vc	v31	.128	2032	0	\n"
+			"ctr	xer	.64	2048	0	# Fixed-Point Exception Register\n"
+			"vcc	vrsave	.32	2056	0	# VR Save Register\n"
+			"ctr	fpscr	.64	2060	0	# Floating-Point Status and Control Register\n"
+			"vcc	vscr	.32	2068	0	# Vector Status and Control Register\n"
+			"ctr	cr	.32	2072	0	# Condition Register\n"
+			"ctr	lr	.64	2076	0	# Link Register\n"
+			"ctr	ctr	.64	2084	0	# Count Register\n"
+			"ctr	tar	.64	2092	0	# Target Address Register\n"
+			"ctr	ppr	.64	2100	0	# Process Priority Register\n"
+			"ctr	dscr	.64	2108	0	# Data Stream Control Register\n"
+			"ctr	bescr	.64	2116	0	# Branch Event Status and Control Register\n"
+			"ctr	ebbhr	.64	2124	0	# Event-Based Branch Handler Register\n"
+			"ctr	ebbrr	.64	2132	0	# Event-Based Branch Return Register\n"
+			"ctr	lpcr	.64	2140	0	# Logical Partitioning Control Register\n"
+			"ctr	hrmor	.64	2148	0	# Hypervisor Real Mode Offset Register\n"
+			"ctr	lpidr	.32	2156	0	# Logical Partition Identification Regiser\n"
+			"ctr	pcr	.64	2160	0	# Processor Compatibility Register\n"
+			"ctr	tir	.64	2168	0	# Thread Identification Register\n"
+			"ctr	urmor	.64	2176	0	# Ultravisor Real Mode Offset Register\n"
+			"ctr	smfctrl	.64	2184	0	# Secure Memory Facility Control Register\n"
+			"ctr	msr	.64	2192	0	# Machine State Register\n"
+			"ctr	pvr	.32	2200	0	# Processor Version Register\n"
+			"ctr	pir	.32	2204	0	# Processor Version Register\n"
+			"ctr	pidr	.32	2208	0	# Processor Identification Register\n"
+			"ctr	ctrl	.32	2212	0	# Control Register\n"
+			"ctr	pspb	.32	2216	0	# Problem State Priority Boost Register\n"
+			"ctr	rpr	.64	2220	0	# Relative Priority Register\n"
+			"ctr	hashkey	.64	2228	0	# Hash Key Register\n"
+			"ctr	hashpkey	.64	2236	0	# Hash Privileged Key Register\n"
+			"ctr	sprg0	.64	2244	0	# Software-use SPR 0\n"
+			"ctr	sprg1	.64	2252	0	# Software-use SPR 1\n"
+			"ctr	sprg2	.64	2260	0	# Software-use SPR 2\n"
+			"ctr	sprg3	.64	2268	0	# Software-use SPR 3\n"
+			"ctr	hsprg0	.64	2276	0	# Hypervisor Software-use SPR 0\n"
+			"ctr	hsprg1	.64	2284	0	# Hypervisor Software-use SPR 1\n"
+			"ctr	usprg0	.64	2292	0	# Ultravisor Software-use SPR 0\n"
+			"ctr	usprg1	.64	2300	0	# Ultravisor Software-use SPR 1\n"
+			"ctr	srr0	.64	2308	0	# Machine Status Save/Restore Register 0\n"
+			"ctr	srr1	.64	2316	0	# Machine Status Save/Restore Register 1\n"
+			"ctr	hsrr0	.64	2324	0	# Hypervisor Machine Status Save/Restore Register 0\n"
+			"ctr	hsrr1	.64	2332	0	# Hypervisor Machine Status Save/Restore Register 1\n"
+			"ctr	usrr0	.64	2340	0	# Hypervisor Machine Status Save/Restore Register 0\n"
+			"ctr	usrr1	.64	2348	0	# Ultravisor Machine Status Save/Restore Register 1\n"
+			"ctr	asdr	.64	2356	0	# Access Segment Descriptor Register\n"
+			"ctr	dar	.64	2364	0	# Data Address Register\n"
+			"ctr	hdar	.64	2372	0	# Hypervisor Data Address Register\n"
+			"ctr	dsisr	.32	2380	0	# Data Storage Interrupt Status Register\n"
+			"ctr	hdsisr	.32	2384	0	# Hypervisor Data Storage Interrupt Status Register\n"
+			"ctr	heir	.64	2388	0	# Hypervisor Emulation Instruction Register\n"
+			"ctr	hmer	.64	2396	0	# Hypervisor Maintenance Exception Register\n"
+			"ctr	hmeer	.64	2404	0	# Hypervisor Maintenance Exception Enable Register\n"
+			"ctr	fscr	.64	2412	0	# Facility Status and Control Register\n"
+			"ctr	hfscr	.64	2420	0	# Hypervisor Facility Status and Control Register\n"
+			"ctr	purr	.64	2428	0	# Process Utilization of Resources Register\n"
+			"ctr	spurr	.64	2436	0	# Scaled Process Utilization of Resources Register\n"
+			"ctr	dexcr	.64	2444	0	# Dynamic Execution Control Register\n"
+			"ctr	hdexcr	.64	2452	0	# Hypervisor Dynamic Execution Control Register\n"
+			"ctr	udexcr	.64	2460	0	# Ultravisor Dynamic Execution Control Register\n"
+			"ctr	cfar	.64	2468	0	# Come-From Address Register\n"
+			"ctr	mmcr0	.64	2476	0	# Monitor Mode Control Register 0\n"
+			"ctr	mmcr1	.64	2484	0	# Monitor Mode Control Register 1\n"
+			"ctr	mmcr2	.64	2492	0	# Monitor Mode Control Register 2\n"
+			"ctr	mmcra	.64	2500	0	# Monitor Mode Control Register A\n"
+			"ctr	siar	.64	2508	0	# Sampled Instruction Address Register\n"
+			"ctr	sdar	.64	2516	0	# Sampled Data Address Register\n"
+			"ctr	sier	.64	2524	0	# Sampled Instruction Event Register\n"
+			"ctr	sier2	.64	2532	0	# Sampled Instruction Event Register 2\n"
+			"ctr	sier3	.64	2540	0	# Sampled Instruction Event Register 3\n"
+			"ctr	mmcr3	.64	2548	0	# Monitor Mode Control Register 2\n"
+			"ctr	dpdes	.64	2556	0	# Directed Privileged Doorbell Exception State Register\n"
+			"ctr	pc	.64	2564	0	# Programm Counter\n"
+			"ctr	cr0	.4	2572	0	# Condition Register Field 0\n"
+			"ctr	cr1	.4	2573	0	# Condition Register Field 1\n"
+			"ctr	cr2	.4	2574	0	# Condition Register Field 2\n"
+			"ctr	cr3	.4	2575	0	# Condition Register Field 3\n"
+			"ctr	cr4	.4	2576	0	# Condition Register Field 4\n"
+			"ctr	cr5	.4	2577	0	# Condition Register Field 5\n"
+			"ctr	cr6	.4	2578	0	# Condition Register Field 6\n"
+			"ctr	cr7	.4	2579	0	# Condition Register Field 7\n"
+			"ctr	ppr32	.32	2580	0	# Process Priority Register 32-bit\n"
+			"flg	so	.1	2584	0	# Summary Overflow\n"
+			"flg	ov	.1	2585	0	# Overflow\n"
+			"flg	ca	.1	2586	0	# Carry\n";
+		return strdup(p);
 	} else {
 		p =
 			"=PC	pc\n"
 			"=SP	r1\n"
-			"=SR	srr1\n" // status register ??
-			"=SN	r0\n" // also for ret
+			"=BP	r31\n"
+			"=SN	r3\n" // also for ret
 			"=A0	r3\n" // also for ret
 			"=A1	r4\n"
 			"=A2	r5\n"
@@ -323,95 +480,254 @@ static char *get_reg_profile(RzAnalysis *analysis) {
 			"=A4	r7\n"
 			"=A5	r8\n"
 			"=A6	r6\n"
-			"gpr	srr0   .64 0   0\n"
-			"gpr	srr1   .64 8   0\n"
-			"gpr	r0   .64 16  0\n"
-			"gpr	r1   .64 24  0\n"
-			"gpr	r2   .64 32  0\n"
-			"gpr	r3   .64 40  0\n"
-			"gpr	r4   .64 48  0\n"
-			"gpr	r5   .64 56  0\n"
-			"gpr	r6   .64 64  0\n"
-			"gpr	r7   .64 72  0\n"
-			"gpr	r8   .64 80  0\n"
-			"gpr	r9   .64 88  0\n"
-			"gpr	r10 .64 96  0\n"
-			"gpr	r11 .64 104 0\n"
-			"gpr	r12 .64 112 0\n"
-			"gpr	r13 .64 120 0\n"
-			"gpr	r14 .64 128 0\n"
-			"gpr	r15 .64 136 0\n"
-			"gpr	r16 .64 144 0\n"
-			"gpr	r17 .64 152 0\n"
-			"gpr	r18 .64 160 0\n"
-			"gpr	r19 .64 168 0\n"
-			"gpr	r20 .64 176 0\n"
-			"gpr	r21 .64 184 0\n"
-			"gpr	r22 .64 192 0\n"
-			"gpr	r23 .64 200 0\n"
-			"gpr	r24 .64 208 0\n"
-			"gpr	r25 .64 216 0\n"
-			"gpr	r26 .64 224 0\n"
-			"gpr	r27 .64 232 0\n"
-			"gpr	r28 .64 240 0\n"
-			"gpr	r29 .64 248 0\n"
-			"gpr	r30 .64 256 0\n"
-			"gpr	r31 .64 264 0\n"
-			"gpr	lr   .64 272 0\n"
-			"gpr	ctr .64 280 0\n"
-			"gpr	msr .64 288 0\n"
-			"gpr	pc   .64 296 0\n"
-			"gpr	cr  .64 304 0\n"
-			"gpr	cr0 .8  304 0\n"
-			"gpr	cr1 .8  305 0\n"
-			"gpr	cr2 .8  306 0\n"
-			"gpr	cr3 .8  307 0\n"
-			"gpr	cr4 .8  308 0\n"
-			"gpr	cr5 .8  309 0\n"
-			"gpr	cr6 .8  310 0\n"
-			"gpr	cr7 .8  311 0\n"
-			"gpr	xer .64 312 0\n"
-			"gpr	mq   .64 320 0\n"
-			"gpr	fpscr  .64 328 0\n"
-			"gpr	vrsave .64 336 0\n"
-			"gpr	pvr .64 344 0\n"
-			"gpr	dccr   .32 352 0\n"
-			"gpr	iccr   .32 356 0\n"
-			"gpr	dear   .32 360 0\n"
-			"gpr	hid0   .64 364 0\n"
-			"gpr	hid1   .64 372 0\n"
-			"gpr	hid2   .64 380 0\n"
-			"gpr	hid3   .64 388 0\n"
-			"gpr	hid4   .64 396 0\n"
-			"gpr	hid5   .64 404 0\n"
-			"gpr	hid6   .64 412 0\n"
-			"gpr	ibat0  .64 420 0\n"
-			"gpr	ibat1  .64 428 0\n"
-			"gpr	ibat2  .64 436 0\n"
-			"gpr	ibat3  .64 444 0\n"
-			"gpr	ibat0l .32 420 0\n"
-			"gpr	ibat1l .32 428 0\n"
-			"gpr	ibat2l .32 436 0\n"
-			"gpr	ibat3l .32 444 0\n"
-			"gpr	ibat0u .32 424 0\n"
-			"gpr	ibat1u .32 432 0\n"
-			"gpr	ibat2u .32 440 0\n"
-			"gpr	ibat3u .32 448 0\n"
-			"gpr	dbat0  .64 456 0\n"
-			"gpr	dbat1  .64 464 0\n"
-			"gpr	dbat2  .64 472 0\n"
-			"gpr	dbat3  .64 480 0\n"
-			"gpr	dbat0l .32 456 0\n"
-			"gpr	dbat1l .32 464 0\n"
-			"gpr	dbat2l .32 472 0\n"
-			"gpr	dbat3l .32 480 0\n"
-			"gpr	dbat0u .32 460 0\n"
-			"gpr	dbat1u .32 468 0\n"
-			"gpr	dbat2u .32 476 0\n"
-			"gpr	dbat3u .32 484 0\n"
-			"gpr	mask   .64 488 0\n"; // not a real register used on complex functions
+			"=OF	ov\n"
+			"=CF	ca\n"
+
+			"gpr	r0	.32	0	0	\n"
+			"gpr	r1	.32	8	0	\n"
+			"gpr	r2	.32	16	0	\n"
+			"gpr	r3	.32	24	0	\n"
+			"gpr	r4	.32	32	0	\n"
+			"gpr	r5	.32	40	0	\n"
+			"gpr	r6	.32	48	0	\n"
+			"gpr	r7	.32	56	0	\n"
+			"gpr	r8	.32	64	0	\n"
+			"gpr	r9	.32	72	0	\n"
+			"gpr	r10	.32	80	0	\n"
+			"gpr	r11	.32	88	0	\n"
+			"gpr	r12	.32	96	0	\n"
+			"gpr	r13	.32	104	0	\n"
+			"gpr	r14	.32	112	0	\n"
+			"gpr	r15	.32	120	0	\n"
+			"gpr	r16	.32	128	0	\n"
+			"gpr	r17	.32	136	0	\n"
+			"gpr	r18	.32	144	0	\n"
+			"gpr	r19	.32	152	0	\n"
+			"gpr	r20	.32	160	0	\n"
+			"gpr	r21	.32	168	0	\n"
+			"gpr	r22	.32	176	0	\n"
+			"gpr	r23	.32	184	0	\n"
+			"gpr	r24	.32	192	0	\n"
+			"gpr	r25	.32	200	0	\n"
+			"gpr	r26	.32	208	0	\n"
+			"gpr	r27	.32	216	0	\n"
+			"gpr	r28	.32	224	0	\n"
+			"gpr	r29	.32	232	0	\n"
+			"gpr	r30	.32	240	0	\n"
+			"gpr	r31	.32	248	0	\n"
+			"vc	vs0	.128	256	0	\n"
+			"vc	vs1	.128	272	0	\n"
+			"vc	vs2	.128	288	0	\n"
+			"vc	vs3	.128	304	0	\n"
+			"vc	vs4	.128	320	0	\n"
+			"vc	vs5	.128	336	0	\n"
+			"vc	vs6	.128	352	0	\n"
+			"vc	vs7	.128	368	0	\n"
+			"vc	vs8	.128	384	0	\n"
+			"vc	vs9	.128	400	0	\n"
+			"vc	vs10	.128	416	0	\n"
+			"vc	vs11	.128	432	0	\n"
+			"vc	vs12	.128	448	0	\n"
+			"vc	vs13	.128	464	0	\n"
+			"vc	vs14	.128	480	0	\n"
+			"vc	vs15	.128	496	0	\n"
+			"vc	vs16	.128	512	0	\n"
+			"vc	vs17	.128	528	0	\n"
+			"vc	vs18	.128	544	0	\n"
+			"vc	vs19	.128	560	0	\n"
+			"vc	vs20	.128	576	0	\n"
+			"vc	vs21	.128	592	0	\n"
+			"vc	vs22	.128	608	0	\n"
+			"vc	vs23	.128	624	0	\n"
+			"vc	vs24	.128	640	0	\n"
+			"vc	vs25	.128	656	0	\n"
+			"vc	vs26	.128	672	0	\n"
+			"vc	vs27	.128	688	0	\n"
+			"vc	vs28	.128	704	0	\n"
+			"vc	vs29	.128	720	0	\n"
+			"vc	vs30	.128	736	0	\n"
+			"vc	vs31	.128	752	0	\n"
+			"vc	vs32	.128	768	0	\n"
+			"vc	vs33	.128	784	0	\n"
+			"vc	vs34	.128	800	0	\n"
+			"vc	vs35	.128	816	0	\n"
+			"vc	vs36	.128	832	0	\n"
+			"vc	vs37	.128	848	0	\n"
+			"vc	vs38	.128	864	0	\n"
+			"vc	vs39	.128	880	0	\n"
+			"vc	vs40	.128	896	0	\n"
+			"vc	vs41	.128	912	0	\n"
+			"vc	vs42	.128	928	0	\n"
+			"vc	vs43	.128	944	0	\n"
+			"vc	vs44	.128	960	0	\n"
+			"vc	vs45	.128	976	0	\n"
+			"vc	vs46	.128	992	0	\n"
+			"vc	vs47	.128	1008	0	\n"
+			"vc	vs48	.128	1024	0	\n"
+			"vc	vs49	.128	1040	0	\n"
+			"vc	vs50	.128	1056	0	\n"
+			"vc	vs51	.128	1072	0	\n"
+			"vc	vs52	.128	1088	0	\n"
+			"vc	vs53	.128	1104	0	\n"
+			"vc	vs54	.128	1120	0	\n"
+			"vc	vs55	.128	1136	0	\n"
+			"vc	vs56	.128	1152	0	\n"
+			"vc	vs57	.128	1168	0	\n"
+			"vc	vs58	.128	1184	0	\n"
+			"vc	vs59	.128	1200	0	\n"
+			"vc	vs60	.128	1216	0	\n"
+			"vc	vs61	.128	1232	0	\n"
+			"vc	vs62	.128	1248	0	\n"
+			"vc	vs63	.128	1264	0	\n"
+			"fpu	f0	.64	1280	0	\n"
+			"fpu	f1	.64	1288	0	\n"
+			"fpu	f2	.64	1296	0	\n"
+			"fpu	f3	.64	1304	0	\n"
+			"fpu	f4	.64	1312	0	\n"
+			"fpu	f5	.64	1320	0	\n"
+			"fpu	f6	.64	1328	0	\n"
+			"fpu	f7	.64	1336	0	\n"
+			"fpu	f8	.64	1344	0	\n"
+			"fpu	f9	.64	1352	0	\n"
+			"fpu	f10	.64	1360	0	\n"
+			"fpu	f11	.64	1368	0	\n"
+			"fpu	f12	.64	1376	0	\n"
+			"fpu	f13	.64	1384	0	\n"
+			"fpu	f14	.64	1392	0	\n"
+			"fpu	f15	.64	1400	0	\n"
+			"fpu	f16	.64	1408	0	\n"
+			"fpu	f17	.64	1416	0	\n"
+			"fpu	f18	.64	1424	0	\n"
+			"fpu	f19	.64	1432	0	\n"
+			"fpu	f20	.64	1440	0	\n"
+			"fpu	f21	.64	1448	0	\n"
+			"fpu	f22	.64	1456	0	\n"
+			"fpu	f23	.64	1464	0	\n"
+			"fpu	f24	.64	1472	0	\n"
+			"fpu	f25	.64	1480	0	\n"
+			"fpu	f26	.64	1488	0	\n"
+			"fpu	f27	.64	1496	0	\n"
+			"fpu	f28	.64	1504	0	\n"
+			"fpu	f29	.64	1512	0	\n"
+			"fpu	f30	.64	1520	0	\n"
+			"fpu	f31	.64	1528	0	\n"
+			"vc	v0	.128	1536	0	\n"
+			"vc	v1	.128	1552	0	\n"
+			"vc	v2	.128	1568	0	\n"
+			"vc	v3	.128	1584	0	\n"
+			"vc	v4	.128	1600	0	\n"
+			"vc	v5	.128	1616	0	\n"
+			"vc	v6	.128	1632	0	\n"
+			"vc	v7	.128	1648	0	\n"
+			"vc	v8	.128	1664	0	\n"
+			"vc	v9	.128	1680	0	\n"
+			"vc	v10	.128	1696	0	\n"
+			"vc	v11	.128	1712	0	\n"
+			"vc	v12	.128	1728	0	\n"
+			"vc	v13	.128	1744	0	\n"
+			"vc	v14	.128	1760	0	\n"
+			"vc	v15	.128	1776	0	\n"
+			"vc	v16	.128	1792	0	\n"
+			"vc	v17	.128	1808	0	\n"
+			"vc	v18	.128	1824	0	\n"
+			"vc	v19	.128	1840	0	\n"
+			"vc	v20	.128	1856	0	\n"
+			"vc	v21	.128	1872	0	\n"
+			"vc	v22	.128	1888	0	\n"
+			"vc	v23	.128	1904	0	\n"
+			"vc	v24	.128	1920	0	\n"
+			"vc	v25	.128	1936	0	\n"
+			"vc	v26	.128	1952	0	\n"
+			"vc	v27	.128	1968	0	\n"
+			"vc	v28	.128	1984	0	\n"
+			"vc	v29	.128	2000	0	\n"
+			"vc	v30	.128	2016	0	\n"
+			"vc	v31	.128	2032	0	\n"
+			"ctr	xer	.64	2048	0	# Fixed-Point Exception Register\n"
+			"vcc	vrsave	.32	2056	0	# VR Save Register\n"
+			"ctr	fpscr	.64	2060	0	# Floating-Point Status and Control Register\n"
+			"vcc	vscr	.32	2068	0	# Vector Status and Control Register\n"
+			"ctr	cr	.32	2072	0	# Condition Register\n"
+			"ctr	lr	.32	2076	0	# Link Register\n"
+			"ctr	ctr	.32	2084	0	# Count Register\n"
+			"ctr	tar	.32	2092	0	# Target Address Register\n"
+			"ctr	ppr	.64	2100	0	# Process Priority Register\n"
+			"ctr	dscr	.64	2108	0	# Data Stream Control Register\n"
+			"ctr	bescr	.64	2116	0	# Branch Event Status and Control Register\n"
+			"ctr	ebbhr	.64	2124	0	# Event-Based Branch Handler Register\n"
+			"ctr	ebbrr	.64	2132	0	# Event-Based Branch Return Register\n"
+			"ctr	lpcr	.64	2140	0	# Logical Partitioning Control Register\n"
+			"ctr	hrmor	.64	2148	0	# Hypervisor Real Mode Offset Register\n"
+			"ctr	lpidr	.32	2156	0	# Logical Partition Identification Regiser\n"
+			"ctr	pcr	.64	2160	0	# Processor Compatibility Register\n"
+			"ctr	tir	.64	2168	0	# Thread Identification Register\n"
+			"ctr	urmor	.64	2176	0	# Ultravisor Real Mode Offset Register\n"
+			"ctr	smfctrl	.64	2184	0	# Secure Memory Facility Control Register\n"
+			"ctr	msr	.64	2192	0	# Machine State Register\n"
+			"ctr	pvr	.32	2200	0	# Processor Version Register\n"
+			"ctr	pir	.32	2204	0	# Processor Version Register\n"
+			"ctr	pidr	.32	2208	0	# Processor Identification Register\n"
+			"ctr	ctrl	.32	2212	0	# Control Register\n"
+			"ctr	pspb	.32	2216	0	# Problem State Priority Boost Register\n"
+			"ctr	rpr	.64	2220	0	# Relative Priority Register\n"
+			"ctr	hashkey	.64	2228	0	# Hash Key Register\n"
+			"ctr	hashpkey	.64	2236	0	# Hash Privileged Key Register\n"
+			"ctr	sprg0	.64	2244	0	# Software-use SPR 0\n"
+			"ctr	sprg1	.64	2252	0	# Software-use SPR 1\n"
+			"ctr	sprg2	.64	2260	0	# Software-use SPR 2\n"
+			"ctr	sprg3	.64	2268	0	# Software-use SPR 3\n"
+			"ctr	hsprg0	.64	2276	0	# Hypervisor Software-use SPR 0\n"
+			"ctr	hsprg1	.64	2284	0	# Hypervisor Software-use SPR 1\n"
+			"ctr	usprg0	.64	2292	0	# Ultravisor Software-use SPR 0\n"
+			"ctr	usprg1	.64	2300	0	# Ultravisor Software-use SPR 1\n"
+			"ctr	srr0	.64	2308	0	# Machine Status Save/Restore Register 0\n"
+			"ctr	srr1	.64	2316	0	# Machine Status Save/Restore Register 1\n"
+			"ctr	hsrr0	.64	2324	0	# Hypervisor Machine Status Save/Restore Register 0\n"
+			"ctr	hsrr1	.64	2332	0	# Hypervisor Machine Status Save/Restore Register 1\n"
+			"ctr	usrr0	.64	2340	0	# Hypervisor Machine Status Save/Restore Register 0\n"
+			"ctr	usrr1	.64	2348	0	# Ultravisor Machine Status Save/Restore Register 1\n"
+			"ctr	asdr	.64	2356	0	# Access Segment Descriptor Register\n"
+			"ctr	dar	.64	2364	0	# Data Address Register\n"
+			"ctr	hdar	.64	2372	0	# Hypervisor Data Address Register\n"
+			"ctr	dsisr	.32	2380	0	# Data Storage Interrupt Status Register\n"
+			"ctr	hdsisr	.32	2384	0	# Hypervisor Data Storage Interrupt Status Register\n"
+			"ctr	heir	.64	2388	0	# Hypervisor Emulation Instruction Register\n"
+			"ctr	hmer	.64	2396	0	# Hypervisor Maintenance Exception Register\n"
+			"ctr	hmeer	.64	2404	0	# Hypervisor Maintenance Exception Enable Register\n"
+			"ctr	fscr	.64	2412	0	# Facility Status and Control Register\n"
+			"ctr	hfscr	.64	2420	0	# Hypervisor Facility Status and Control Register\n"
+			"ctr	purr	.64	2428	0	# Process Utilization of Resources Register\n"
+			"ctr	spurr	.64	2436	0	# Scaled Process Utilization of Resources Register\n"
+			"ctr	dexcr	.64	2444	0	# Dynamic Execution Control Register\n"
+			"ctr	hdexcr	.64	2452	0	# Hypervisor Dynamic Execution Control Register\n"
+			"ctr	udexcr	.64	2460	0	# Ultravisor Dynamic Execution Control Register\n"
+			"ctr	cfar	.64	2468	0	# Come-From Address Register\n"
+			"ctr	mmcr0	.64	2476	0	# Monitor Mode Control Register 0\n"
+			"ctr	mmcr1	.64	2484	0	# Monitor Mode Control Register 1\n"
+			"ctr	mmcr2	.64	2492	0	# Monitor Mode Control Register 2\n"
+			"ctr	mmcra	.64	2500	0	# Monitor Mode Control Register A\n"
+			"ctr	siar	.64	2508	0	# Sampled Instruction Address Register\n"
+			"ctr	sdar	.64	2516	0	# Sampled Data Address Register\n"
+			"ctr	sier	.64	2524	0	# Sampled Instruction Event Register\n"
+			"ctr	sier2	.64	2532	0	# Sampled Instruction Event Register 2\n"
+			"ctr	sier3	.64	2540	0	# Sampled Instruction Event Register 3\n"
+			"ctr	mmcr3	.64	2548	0	# Monitor Mode Control Register 2\n"
+			"ctr	dpdes	.64	2556	0	# Directed Privileged Doorbell Exception State Register\n"
+			"ctr	pc	.64	2564	0	# Programm Counter\n"
+			"ctr	cr0	.4	2572	0	# Condition Register Field 0\n"
+			"ctr	cr1	.4	2573	0	# Condition Register Field 1\n"
+			"ctr	cr2	.4	2574	0	# Condition Register Field 2\n"
+			"ctr	cr3	.4	2575	0	# Condition Register Field 3\n"
+			"ctr	cr4	.4	2576	0	# Condition Register Field 4\n"
+			"ctr	cr5	.4	2577	0	# Condition Register Field 5\n"
+			"ctr	cr6	.4	2578	0	# Condition Register Field 6\n"
+			"ctr	cr7	.4	2579	0	# Condition Register Field 7\n"
+			"ctr	ppr32	.32	2580	0	# Process Priority Register 32-bit\n"
+			"flg	so	.1	2584	0	# Summary Overflow\n"
+			"flg	ov	.1	2585	0	# Overflow\n"
+			"flg	ca	.1	2586	0	# Carry\n";
+		return strdup(p);
 	}
-	return strdup(p);
 }
 
 static int analop_vle(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
@@ -638,7 +954,9 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 	n = cs_disasm(handle, (const ut8 *)buf, len, addr, 1, &insn);
 	if (n < 1) {
 		op->type = RZ_ANALYSIS_OP_TYPE_ILL;
+		op->il_op = rz_il_op_new_empty();
 	} else {
+		op->il_op = rz_ppc_cs_get_il_op(handle, insn, mode);
 		if (mask & RZ_ANALYSIS_OP_MASK_DISASM) {
 			op->mnemonic = strdup(insn->mnemonic);
 		}
@@ -1300,20 +1618,35 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 	return op->size;
 }
 
-static int archinfo(RzAnalysis *a, int q) {
-	if (a->cpu && !strncmp(a->cpu, "vle", 3)) {
-		return 2;
+static int archinfo(RzAnalysis *a, RzAnalysisInfoType query) {
+	bool is_vle = a && a->cpu && !strncmp(a->cpu, "vle", 3);
+
+	switch (query) {
+	case RZ_ANALYSIS_ARCHINFO_MIN_OP_SIZE:
+		return is_vle ? 2 : 4;
+	case RZ_ANALYSIS_ARCHINFO_MAX_OP_SIZE:
+		return 4;
+	case RZ_ANALYSIS_ARCHINFO_CAN_USE_POINTERS:
+		return true;
+	default:
+		return -1;
 	}
-	return 4;
 }
 
-static RzList *analysis_preludes(RzAnalysis *analysis) {
+static RzList /*<RzSearchKeyword *>*/ *analysis_preludes(RzAnalysis *analysis) {
 #define KW(d, ds, m, ms) rz_list_append(l, rz_search_keyword_new((const ut8 *)d, ds, (const ut8 *)m, ms, NULL))
 	RzList *l = rz_list_newf((RzListFree)rz_search_keyword_free);
 	KW("\x7c\x08\x02\xa6", 4, NULL, 0);
 	return l;
 }
 
+static RzAnalysisILConfig *il_config(RzAnalysis *analysis) {
+	if (analysis->bits == 64) {
+		return rz_ppc_cs_64_il_config(analysis->big_endian);
+	}
+	return rz_ppc_cs_32_il_config(analysis->big_endian);
+}
+
 RzAnalysisPlugin rz_analysis_plugin_ppc_cs = {
 	.name = "ppc",
 	.desc = "Capstone PowerPC analysis",
@@ -1325,6 +1658,7 @@ RzAnalysisPlugin rz_analysis_plugin_ppc_cs = {
 	.preludes = analysis_preludes,
 	.op = &analop,
 	.get_reg_profile = &get_reg_profile,
+	.il_config = il_config,
 };
 
 #ifndef RZ_PLUGIN_INCORE
