@@ -4,6 +4,7 @@
 #include <rz_core.h>
 
 #define GO_MAX_STRING_SIZE 0x4000
+#define GO_MAX_TABLE_SIZE  0x10000
 
 #define GO_1_2  (12)
 #define GO_1_16 (116)
@@ -484,29 +485,26 @@ RZ_API bool rz_core_analysis_recover_golang_functions(RzCore *core) {
 }
 
 static bool add_new_bin_string(RzCore *core, char *string, ut64 vaddr, ut32 size) {
-	RzListIter *it;
+	ut32 ordinal = 0;
 	RzBinString *bstr;
-	RzBinFile *bf = rz_bin_cur(core->bin);
+	RzBin *bin = core->bin;
+	RzBinFile *bf = rz_bin_cur(bin);
 	if (!bf || !bf->o || !bf->o->strings) {
 		free(string);
 		return false;
 	}
-	ut64 paddr = rz_io_v2p(core->io, vaddr);
 
-	rz_list_foreach (bf->o->strings, it, bstr) {
-		if (bstr->vaddr == vaddr && bstr->size == size) {
-			free(string);
-			return true;
-		}
-		ut64 end = bstr->vaddr + bstr->size;
-		if (vaddr >= bstr->vaddr && (vaddr + size) < end) {
-			RzListIter *prev = it->p;
-			rz_list_delete(bf->o->strings, it);
-			it = prev ? prev : bf->o->strings->head;
-			break;
-		}
+	bstr = rz_bin_object_get_string_at(bf->o, vaddr, true);
+	if (bstr && bstr->vaddr == vaddr && bstr->size == size) {
+		free(string);
+		return true;
 	}
 
+	const RzList *strings = rz_bin_object_get_strings(bf->o);
+	ordinal = rz_list_length(strings);
+
+	ut64 paddr = rz_io_v2p(core->io, vaddr);
+
 	bstr = RZ_NEW0(RzBinString);
 	if (!bstr) {
 		RZ_LOG_ERROR("Failed allocate new go string\n");
@@ -515,12 +513,12 @@ static bool add_new_bin_string(RzCore *core, char *string, ut64 vaddr, ut32 size
 	}
 	bstr->paddr = paddr;
 	bstr->vaddr = vaddr;
-	bstr->ordinal = rz_list_length(bf->o->strings);
+	bstr->ordinal = ordinal;
 	bstr->length = bstr->size = size;
 	bstr->string = string;
-	bstr->type = RZ_BIN_STRING_ENC_UTF8;
-	if (!rz_list_append(bf->o->strings, bstr)) {
-		RZ_LOG_ERROR("Failed append new go string to strings list\n");
+	bstr->type = RZ_STRING_ENC_UTF8;
+	if (!rz_bin_string_database_add(bf->o->strings, bstr)) {
+		RZ_LOG_ERROR("Failed append new go string to strings database\n");
 		rz_bin_string_free(bstr);
 		return false;
 	}
@@ -533,6 +531,12 @@ static bool recover_string_at(GoStrRecover *ctx, ut64 str_addr, ut64 str_size) {
 		return false;
 	}
 
+	// skip possible pointers that matches to symbols flags, because these are already handled.
+	RzFlagItem *fi = rz_flag_get_by_spaces(ctx->core->flags, str_addr, RZ_FLAGS_FS_SYMBOLS, NULL);
+	if (fi && !strncmp(fi->name, "sym.", 4)) {
+		return false;
+	}
+
 	RzBinObject *obj = rz_bin_cur_object(ctx->core->bin);
 	if (!obj || !rz_bin_get_section_at(obj, str_addr, true)) {
 		// skip any possible string from invalid sections.
@@ -648,7 +652,7 @@ static bool go_is_sign_match(GoStrRecover *ctx, GoStrInfo *info, GoSignature *si
 static ut32 decode_one_opcode_size(GoStrRecover *ctx) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(ctx->core->analysis, &aop, ctx->pc, ctx->bytes, ctx->size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(ctx->core->analysis, &aop, ctx->pc, ctx->bytes, ctx->size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return 0;
 	}
@@ -677,7 +681,7 @@ static bool decode_from_table(RzCore *core, GoStrInfo *info, ut64 pc, const ut8
 static bool decode_val_set_size(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -689,7 +693,7 @@ static bool decode_val_set_size(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 static bool decode_val_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -701,7 +705,7 @@ static bool decode_val_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 static bool decode_val_add_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -713,7 +717,7 @@ static bool decode_val_add_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 static bool decode_ptr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -725,7 +729,7 @@ static bool decode_ptr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 static bool decode_disp_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -1042,7 +1046,7 @@ static bool decode_ldr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 	ut64 addr = 0;
 
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -1115,7 +1119,7 @@ static ut32 golang_recover_string_arm32(GoStrRecover *ctx) {
 static bool decode_lui_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -1499,7 +1503,7 @@ static ut32 golang_recover_string_ppc64(GoStrRecover *ctx) {
 static bool decode_auipc_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
 	RzAnalysisOp aop;
 	rz_analysis_op_init(&aop);
-	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_DISASM) < 1) {
+	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
 		return false;
 	}
@@ -1608,6 +1612,74 @@ static ut32 golang_recover_string_riscv64(GoStrRecover *ctx) {
 	return 4;
 }
 
+// Sometimes the data-structures has strings, but these are stored in tables where
+// the first offset is always the pointer to the squashed strings and the next word
+// is the size of the string
+static void core_recover_golang_strings_from_data_pointers(RzCore *core, GoStrRecover *ctx) {
+	rz_core_notify_begin(core, "Recovering go strings from bin maps");
+
+	RzAnalysis *analysis = core->analysis;
+	const ut32 word_size = analysis->bits / 8;
+	RzListIter *iter;
+	RzBinMap *map;
+	ut8 *buffer = NULL;
+	ut64 string_addr, string_size;
+	RzBinObject *object = rz_bin_cur_object(core->bin);
+	RzList *map_list = object ? rz_bin_object_get_maps(object) : NULL;
+	if (!map_list) {
+		RZ_LOG_ERROR("Failed to get the RzBinMap list\n");
+		goto end;
+	}
+
+	buffer = malloc(GO_MAX_TABLE_SIZE);
+	if (!buffer) {
+		RZ_LOG_ERROR("Failed to allocate table buffer\n");
+		goto end;
+	}
+
+	rz_list_foreach (map_list, iter, map) {
+		if (!rz_bin_map_is_data(map) || map->psize < (word_size * 2)) {
+			continue;
+		}
+
+		ut64 current = map->vaddr;
+		ut64 end = map->vaddr + map->psize;
+
+		do {
+			size_t length = RZ_MIN((end - current), GO_MAX_TABLE_SIZE);
+			if (length < (word_size * 2)) {
+				break;
+			}
+
+			if (rz_io_nread_at(core->io, current, buffer, length) < 0) {
+				RZ_LOG_ERROR("Failed to read map at address %" PFMT64x "\n", current);
+				break;
+			}
+
+			length -= word_size;
+			for (size_t i = 0; i < length; i += word_size) {
+				string_addr = rz_read_ble(buffer + i, analysis->big_endian, analysis->bits);
+				string_size = rz_read_ble(buffer + i + word_size, analysis->big_endian, analysis->bits);
+				if (!string_addr || !string_size) {
+					continue;
+				} else if (word_size == sizeof(ut32) && string_addr == UT32_MAX) {
+					continue;
+				} else if (word_size == sizeof(ut64) && string_addr == UT64_MAX) {
+					continue;
+				}
+				if (recover_string_at(ctx, string_addr, string_size)) {
+					rz_analysis_xrefs_set(analysis, current + i, string_addr, RZ_ANALYSIS_XREF_TYPE_STRING);
+				}
+			}
+			current += length;
+		} while (current < end);
+	}
+
+end:
+	free(buffer);
+	rz_core_notify_done(core, "Recovering go strings from bin maps");
+}
+
 /**
  * \brief      Attempts to recover all golang string
  *
@@ -1615,7 +1687,6 @@ static ut32 golang_recover_string_riscv64(GoStrRecover *ctx) {
  */
 RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 	rz_return_if_fail(core && core->analysis && core->analysis->fcns && core->io);
-	rz_core_notify_begin(core, "Analyze all instructions to recover all strings used in sym.go.*");
 
 	const char *asm_arch = rz_config_get(core->config, "asm.arch");
 	ut32 asm_bits = rz_config_get_i(core->config, "asm.bits");
@@ -1628,6 +1699,9 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 	GoStrRecover ctx = { 0 };
 	ctx.core = core;
 
+	core_recover_golang_strings_from_data_pointers(core, &ctx);
+
+	rz_core_notify_begin(core, "Analyze all instructions to recover all strings used in sym.go.*");
 	if (!strcmp(asm_arch, "x86")) {
 		switch (asm_bits) {
 		case 32:
@@ -1719,4 +1793,4 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 
 	rz_core_notify_done(core, "Analyze all instructions to recover all strings used in sym.go.*");
 	rz_core_notify_done(core, "Recovered %d strings from the sym.go.* functions.", ctx.n_recovered);
-}
\ No newline at end of file
+}
