@@ -7,6 +7,7 @@
 
 #include <rz_types.h>
 #include <rz_parse.h>
+#include <rz_lib.h>
 #include <config.h>
 
 RZ_LIB_VERSION(rz_parse);
@@ -19,7 +20,7 @@ RZ_API RzParse *rz_parse_new(void) {
 	if (!p) {
 		return NULL;
 	}
-	p->parsers = rz_list_newf(NULL); // memleak
+	p->parsers = rz_list_new();
 	if (!p->parsers) {
 		rz_parse_free(p);
 		return NULL;
@@ -31,28 +32,48 @@ RZ_API RzParse *rz_parse_new(void) {
 	p->subtail = false;
 	p->minval = 0x100;
 	p->localvar_only = false;
-	for (i = 0; parse_static_plugins[i]; i++) {
-		rz_parse_add(p, parse_static_plugins[i]);
+	for (i = 0; i < RZ_ARRAY_SIZE(parse_static_plugins); i++) {
+		rz_parse_plugin_add(p, parse_static_plugins[i]);
 	}
 	return p;
 }
 
 RZ_API void rz_parse_free(RzParse *p) {
+	if (!p) {
+		return;
+	}
+	RzListIter *it, *tmp;
+	RzParsePlugin *plugin;
+	rz_list_foreach_safe (p->parsers, it, tmp, plugin) {
+		if (plugin->fini) {
+			plugin->fini(p, p->user);
+		}
+	}
 	rz_list_free(p->parsers);
 	free(p);
 }
 
-RZ_API bool rz_parse_add(RzParse *p, RzParsePlugin *foo) {
+RZ_API bool rz_parse_plugin_add(RzParse *p, RZ_NONNULL RzParsePlugin *plugin) {
+	rz_return_val_if_fail(p && plugin, false);
+
 	bool itsFine = true;
-	if (foo->init) {
-		itsFine = foo->init(p, p->user);
+	if (plugin->init) {
+		itsFine = plugin->init(p, p->user);
 	}
 	if (itsFine) {
-		rz_list_append(p->parsers, foo);
+		rz_list_append(p->parsers, plugin);
 	}
 	return true;
 }
 
+RZ_API bool rz_parse_plugin_del(RzParse *p, RZ_NONNULL RzParsePlugin *plugin) {
+	rz_return_val_if_fail(p && plugin, false);
+	if (plugin->fini && !plugin->fini(p, p->user)) {
+		return false;
+	}
+	return rz_list_delete_data(p->parsers, plugin);
+}
+
 RZ_API bool rz_parse_use(RzParse *p, const char *name) {
 	RzListIter *iter;
 	RzParsePlugin *h;
